/*
 * Author: Samy Al Bahra  <samy@kerneled.org>
 * Date  : Mon Dec 30 02:23:40 2002
 * Copyright(C) 2002-2003 Samy Al Bahra
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: interface.c,v 1.14 2004/01/05 11:11:35 samy Exp $
 */

#include <errno.h>
#include <curses.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <config.h>
#include <headline.h>
#include <interface.h>
#include <wstring.h>

int sigwinch;

/* Resize signal handler */
void
iresize(int sig)
{
	delwin(resources);
	delwin(title);
	delwin(headline);
	delwin(prompt);
	delwin(description);
	endwin();

	sigwinch = 1;
	
	initi();
	erase();
	rmenu(NULL, -1);
}

/* Initialize ncurses interface */
int
initi(void)
{
	if (initscr() == NULL) {
		perror("Could not initialize ncurses interface");
		return 0;
	}

	getmaxyx(stdscr, r, c);

	if (c < 80 || r < 24) {
		if (!sigwinch) endwin();
		fprintf(stderr, "wnews requires atleast an 24x80 character terminal (%d,%d)\n", r, c);
		return 0;
	}

	if ((resources = newwin(r, 16, 0, 0)) == NULL) {
		endwin();
		fprintf(stderr, "Could not create resources window\n");
		return 0;
	}
	if ((title = newwin(3, c - 16, 0, 16)) == NULL) {
		delwin(resources);
		endwin();
		fprintf(stderr, "Could not create title window\n");
		return 0;
	}
	if ((headline = newwin(r - 3, c - 16, 3, 16)) == NULL) {
		delwin(resources);
		delwin(title);
		endwin();
		fprintf(stderr, "Could not create headlines window\n");
		return 0;
	}
	if ((prompt = newwin(8, 20, 0, 0)) == NULL) {
		delwin(resources);
		delwin(title);
		delwin(headline);
		delwin(prompt);
		endwin();
		fprintf(stderr, "Could not create prompt window\n");
		return 0;
	}
	if ((description = newwin(15, 20, 0, 0)) == NULL) {
		delwin(resources);
		delwin(title);
		delwin(headline);
		delwin(prompt);
		endwin();
		fprintf(stderr, "Could not create description window\n");
		return 0;
	}
	ftouch(resources);
	ftouch(title);
	wnoutrefresh(resources);
	wnoutrefresh(title);
	wnoutrefresh(headline);
	doupdate();
	return 1;
}

/* Quit signal handler */
void
iquit(int sig)
{
	delwin(resources);
	delwin(title);
	delwin(headline);
	delwin(prompt);
	delwin(description);
	clear();
	refresh();
	endwin();
	exit(-sig);
}

/* Displays prompt to screen, waits for key press */
void
wprompt(char *fmt, ...)
{
	va_list ap;
	char *buf;
	int len;

	if (!fmt)
		return;

	va_start(ap, fmt);
	if (!(len = vfsalloc(fmt, ap)))
		return;
	va_end(ap);
	
	if (!(buf = malloc(len)))
		return;

	va_start(ap, fmt);		
	vsprintf(buf, fmt, ap);
	va_end(ap);

	werase(title);
	ftouch(title);
	wattron(title, A_BOLD | A_BLINK);
	mvwaddstr(title, 1, 1, buf);
	wattroff(title, A_BOLD | A_BLINK);
	wnoutrefresh(title);
	doupdate();
	getchar();
	werase(title);
	wnoutrefresh(title);
	doupdate();
}

/* Display status information in title bar */
void
wstat(char *fmt, ...)
{
	va_list ap;
	char *buf;
	int len;

	if (!fmt)
		return;

	va_start(ap, fmt);
	if (!(len = vfsalloc(fmt, ap)))
		return;
	va_end(ap);
	
	if (!(buf = malloc(len)))
		return;

	va_start(ap, fmt);		
	vsprintf(buf, fmt, ap);
	va_end(ap);

	werase(title);
	wattron(title, A_BOLD);
	mvwaddstr(title, 1, 1, buf);
	ftouch(title);
	wattroff(title, A_BOLD);
	wnoutrefresh(title);
	doupdate();
}

/* Creates border for window */
void
ftouch(WINDOW * win)
{
	if (ASCII)
		wborder(win, '|', '|', '-', '-', ' ', ' ', ' ', ' ');
	else
		wborder(win, ACS_VLINE, ACS_VLINE, ACS_HLINE, ACS_HLINE,
			     ACS_ULCORNER, ACS_URCORNER, ACS_LLCORNER,
			     ACS_LRCORNER);
}

/* Provides interface to resources list window */
int
rmenu(struct resource ** rarray, int index)
{
	static int i, cursor, current;
	int key = -1, scroll = 0, count = 0;

	if (!cursor)
		cursor = 1;
	
	curs_set(0);
	keypad(resources, TRUE);
	nodelay(resources, FALSE);
	cbreak();
	noecho();
	scroll = index / (r - 2);

	for (;;) {
		werase(resources);
		if (key == KEY_DOWN || tolower(key) == tolower(AUD)) {
			if (cursor + 1 < r - 1 && current + 1 <= index) {
				cursor++;
				current++;
			}
		} else if (key == KEY_UP || tolower(key) == tolower(AUP)) {
			if (cursor - 1 > 0 && current - 1 >= 0) {
				cursor--;
				current--;
			}
		} else if (key == KEY_RIGHT || tolower(key) == tolower(ARIGHT)) {
			i = (i + 1) * (r - 2) > index ? i : i + 1;
			current = i * (r - 2);
			cursor = 1;
		} else if (key == KEY_LEFT || tolower(key) == tolower(ALEFT)) {
			i = i - 1 < 0 ? 0 : i - 1;
			current = i * (r - 2);
			cursor =  1;
		} else if (key == KEY_ENTER || key == KEY_SPACE || tolower(key) == tolower(AENTER) || index < 0) {
			count = i * (r - 2);
			for (key = 1; count <= index && key < r; key++, count++) {
				if (key == cursor)
					wattron(resources, A_BOLD);

				mvwaddstr(resources, key, 2, rarray[count]->title);

				if (key == cursor)
					wattroff(resources, A_BOLD);
			}
	
			ftouch(resources);
			wnoutrefresh(resources);
			doupdate();
			return current;
		} else if (key == KEY_PPAGE || tolower(key) == tolower(APUP)) {
			if (cursor - 5 > 0 && current - 5 >= 0) {
				cursor -= 5;
				current -= 5;
			} else {
				cursor = 1;
				current = i * (r - 2);
			}
		} else if (key == KEY_NPAGE || tolower(key) == tolower(APDN)) {
			if (cursor + 5 < r - 1 && current + 5 <= index) {
				cursor += 5;
				current += 5;
			} else {
				current = i < scroll ? i * (r - 2) + (r - 3) : current;
				cursor = i < scroll ? r - 2 : cursor;
			}
		} else if (tolower(key) == tolower(AURL))
			wprompt("http://%s", rarray[current]->address);
		else if (key == KEY_TAB || tolower(key) == tolower(ATAB))
			return -1;
		else if (tolower(key) == tolower(AQUIT))
			iquit(0);
		if (current > index)
			current = index;
		else if (current < 0)
			current = 0;
		
		count = i * (r - 2);
		for (key = 1; count <= index && key < r; key++, count++) {
			if (key == cursor) {
				wattron(resources, A_REVERSE);
				mvwaddstr(resources, cursor, 1, "              ");
			}
			mvwaddstr(resources, key, 2, rarray[count]->title);
			if (key == cursor)
				wattroff(resources, A_REVERSE);
		}

		ftouch(resources);
		
		if (i > 0) {
			wattron(resources, A_BOLD);
			mvwaddstr(resources, r - 1, 5, "previous...");
			wattroff(resources, A_BOLD);
		}
		else if (i < scroll) {
			wattron(resources, A_BOLD);
			mvwaddstr(resources, r- 1, 9, "next...");
			wattroff(resources, A_BOLD);
		}
		
		werase(title);
		mvwaddstr(title, 1, 1, rarray[current]->desc);
		ftouch(title);
		wnoutrefresh(title);
		wnoutrefresh(resources);
		doupdate();
		key = wgetch(resources);
	}
}

/* Returns an address to given line */
char *
dline(const char *text, int line)
{
	int x = 0;

	for (x = 0; x < strlen(text) && line; x++)
		if (text[x] == '\n')
			line--;

	return (char *)text + x;
}

/* Provides interface to main interface window */
int
hmenu(struct content ** carray, int numc)
{
	char *desc, *command;
	int current = 0, key = 0, y = 0, start = 0;

	curs_set(0);
	keypad(headline, TRUE);
	nodelay(headline, FALSE);
	cbreak();
	noecho();
	wattroff(title, A_BOLD | A_BLINK);

	for (;;) {
		desc = dline(carray[current]->desc, y);
		werase(title);
		werase(headline);
		mvwaddstr(headline, 0, 0, desc);
		wattron(title, A_BOLD);
		mvwaddstr(title, 1, 1, carray[current]->headline);
		wattroff(title, A_BOLD);
		ftouch(title);
		wnoutrefresh(title);
		wnoutrefresh(headline);
		doupdate();

		key = wgetch(headline);

		if (key == KEY_LEFT || tolower(key) == tolower(ALEFT)) {
			current = (current - 1 < 0 ? 0 : current - 1);
			y = 0;
		} else if (key == KEY_RIGHT || tolower(key) == tolower(ARIGHT)) {
			current = (current + 1 > (numc - 1) ? numc - 1 : current + 1);
			y = 0;
		} else if (key == KEY_UP || tolower(key) == tolower(AUP))
			y = (y - 1 > 0) ? y - 1 : 0;
		else if (key == KEY_DOWN || tolower(key) == tolower(AUD))
			++y;
		else if (key == KEY_SPACE || tolower(key) == tolower(AURL)) {
			if (strlen(carray[current]->link) > c - 18) {
				for(start = 0;;) {
					command = strgrab(carray[current]->link + start, carray[current]->link + c - 18);

					if (!command)
						break;

					wstat(command);
					free(command);

					key = wgetch(headline);

					if (key == KEY_RIGHT) {
						if (strlen((carray[current]->link + start + 1)) <= c - 16)
							continue;
						start++;
					} else if (key == KEY_LEFT) {
						if (start)
							start--;
					} else if (key == KEY_SPACE) {
							break;
						}
				}
			} else
				wprompt(carray[current]->link);
		} else if (tolower(key) == tolower(KEY_ENTER) || tolower(key) == tolower(ABROWSER)) {
			if (!browser)
				wprompt("No browser was defined");

			if (!(command = malloc(strlen(browser) + strlen(carray[current]->link) + 1))) {
				wprompt("Could not allocate memory for browser execution");
				break;
			}
			sprintf(command, browser, carray[current]->link);
			system(command);
			free(command);
		} else if (key == KEY_TAB || tolower(key) == tolower(ATAB))
			return -1;
		else if (tolower(key) == tolower(AQUIT))
			iquit(0);
	}

}
