/*
 * Author: Samy Al Bahra  <samy@kerneled.org> 
 * Date  : Mon Dec 30 02:23:40 2002
 * Copyright(C) 2002-2003 Samy Al Bahra
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: string.c,v 1.12 2003/12/27 21:52:47 samy Exp $
 */

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <config.h>
#include <error.h>
#include <headline.h>
#include <wstring.h>


/* Create an array of pointers to linked list items (size number of pointers) */
struct content **
chash(struct content * clist, int index)
{
	struct content *next, **carray;
	int x = 0;

	if (index < 0)
		return NULL;

	if (!(carray = malloc(sizeof(struct content *) * index + 1))) {
		werror("Could not allocate memory for content linked list hash");
		return NULL;
	}
	if (!clist)
		return NULL;

	next = clist;
	while (x <= index) {
		if (next->desc && next->headline) {
			carray[x] = next;
			next = next->next;
			x++;
		}
	}

	return carray;
}

/* Free an array of content structures */
inline int
contfree(struct content ** carray, int index)
{
	int x;

	for (x = 0; x < index; x++) {
		free(carray[x]->desc);
		free(carray[x]->link);
		free(carray[x]->headline);
		free(carray[x]);
	}

	free(carray);
	return 0;
}

/* Create an array of pointers to linked list items (size number of pointers) */
struct resource **
rhash(struct resource * rlist, int index)
{
	struct resource *next, **rarray;

	if (!(rarray = malloc(sizeof(struct resource *) * index + 1))) {
		werror("Could not allocate memory for resources linked list hash");
		return NULL;
	}
	if (!rlist)
		return (struct resource **) NULL;

	next = rlist;
	while (index >= 0) {
		rarray[index] = next;
		next = next->next;
		index--;
	}

	return rarray;
}

/* Allocate memory for format string buffers */
int
vfsalloc(char *fmt, va_list ap)
{
	char *s;
	int i, n = 0, len = 0;

	if (!fmt)
		return 0;

	s = strchr(fmt, '%');

	while (s) {
		++s;
		++n;

		switch (*s) {
		case 'd':
			i = va_arg(ap, int);
			len += 1 + sizeof(int)<<1;
			break;
		case 's':
			len += strlen(va_arg(ap, char *));
			break;
		}
		s = strchr(s, '%');
	}

	len += strlen(fmt) - (2 * n);

	return len;
}

/* Ends string after n characters */
char *
strchop(char *text, int n)
{
	if (strlen(text) > n)
		text[n] = '\0';

	return text;
}

/* Return a new instance of text with HTML characters filtered */
char *
strhtml(const char *text)
{
	unsigned int x, y, l, size, rx;
	char *html;

	if (!text || !(*text))
		return (char *)text;

	l = strlen(text);

	if (!(html = malloc(l + 1))) {
		werror("Could not allocate memory for parsing HTML buffer");
		return NULL;
	}

	for (x = rx = 0; x < l; x++, rx++) {
		if (text[x] == '&') {
			x++;
			for (y = x; y < l; y++) {
				if (text[y] == ';')
					break;
			}
			size = y - x;
			if (size < 2) {
				x = y;
				continue;
			} else {
				if (!strncasecmp(text + x, "quot", 4))
					html[rx] = '"';
				else if (!strncasecmp(text + x, "gt", 2))
					html[rx] = '>';
				else if (!strncasecmp(text + x, "lt", 2))
					html[rx] = '<';
				else if (!strncasecmp(text + x, "amp", 3))
					html[rx] = '&';
				else if (!strncasecmp(text + x, "apos", 4))
					html[rx] = '\'';
	
				x = y;
			}
		} else
			html[rx] = text[x];
	}
	html[rx] = '\0';
	return html;
}

/* Align text to the nearest space to margin */
char *
strcol(const char *text, int margin)
{
	char *ftext;
	unsigned int x, y, col, size;

	if (!text)
		return NULL;

	size = strlen(text);

	if (!(ftext = strdup(text))) {
		werror("Could not allocate memory for text formatting");
		return NULL;
	}

	if (strchr(text, '\n')) {
		for (x = col = 0; x < strlen(text); x++, col++)
			if (ftext[x] == '\t')
				col += 8;
			if (ftext[x] == '\n') {
				if (col > margin) {
					ftext[x] = ' ';
					col = 0;
				} else
					col = -1;
			}
	}

	if (!col)
		return (char *)text;

	for (x = col = 0; x < size; x++, col++) {
		if (ftext[x] == '\n')
			col = 0;
		
		if (col >= margin) {
			if (ftext[x] != ' ' || ftext[x] != '\t' || ftext[x] != '\n') {
				for (y = x; y >= 0; y--) {
					if (ftext[y] == ' ' || ftext[y] == '\t' || ftext[y] == '\n') {
						ftext[y] = '\n';
						break;
					}
				}
			} else {
				ftext[x] = '\n';
			}
			col = 0;
		}
	}
	ftext[size] = '\0';

	return ftext;
}

/* Remove all new lines in a string */
char *
wstripnl(char *strip)
{
	unsigned int x;

	if (!strip)
		return NULL;

	for (x = 0; x < strlen(strip); x++) {
		if (strip[x] == '\n')
			strip[x] = ' ';
	}

	return strip;
}

/* Trim all whitespaces left of a string */
char *
ltrim(char *trim)
{
	unsigned int x;

	if (!trim)
		return NULL;

	for (x = 0; x < strlen(trim); x++) {
		if (trim[x] != ' ' && trim[x] != '\n' && trim[x] != '\t')
			break;
	}

	return trim + x;
}

/* Trim all whitespaces right of a string */
char *
rtrim(char *trim)
{
	unsigned int x;

	if (!trim)
		return NULL;
	for (x = strlen(trim) - 1; x > 0; x--) {
		if (trim[x] != ' ' && trim[x] != '\n' && trim[x] != '\t')
			break;
	}
	trim[x + 1] = '\0';

	return trim;
}

/* Return string in between s and e */
char *
strgrab(char *s, char *e)
{
	unsigned int len;
	char *n;

	if (s > e) {
		werror("strgrab() was given incorrect string");
		return NULL;
	}
	len = e - s + 1;
	if (!(n = malloc(len + 1))) {
		werror("Could not allocate memory for parsing");
		return NULL;
	}
	strncpy(n, s, len);
	n[len] = '\0';

	return n;
}


/* Returns uncompressed buffer */
char *
strgunzip(const char *buffer, int clen)
{
	char *template, *dbuffer, *command;
	unsigned int i = 0;
	FILE *fp;
	int fd;

	if (!buffer)
		return NULL;

	if (!(template = malloc(strlen(tmpdir) + 15))) {
		werror("Could not allocate memory to create temporary file");
		return NULL;
	}
	sprintf(template, "%s/wnews.XXXXXX", tmpdir);

	if (!(command = malloc(strlen(gzip) + strlen(template) + 1))) {
		werror("Could not allocate memory for decompression command");
		return NULL;
	}
	if ((fd = mkstemp(template)) == -1) {
		werror("Could not generate temporary file for decompression");
		free(template);
		return NULL;
	}
	if (write(fd, buffer, clen) < clen) {
		werror("Could not write buffer to temporary file: %s", strerror(errno));
		free(template);
		fclose(fp);
		return NULL;
	}
	/* Let the race begin! */
	close(fd);
	sprintf(command, gzip, template);

	if ((fp = popen(command, "r")) == NULL) {
		werror("Could not establish read-only pipe");
		return NULL;
	}
	if (!(dbuffer = malloc(MBLOCK + 1))) {
		werror("Could not allocate memory for decompression buffer");
		return NULL;
	}
	memset(dbuffer, 0, MBLOCK + 1);

	while (fread((dbuffer + (MBLOCK * i)), MBLOCK, 1, fp) == MBLOCK) {
		if (!(dbuffer = realloc(dbuffer, MBLOCK * (++i)))) {
			werror("Could not enlarge memory for decompression buffer");
			pclose(fp);
			free(command);
			return NULL;
		}
	}

	pclose(fp);

	if (unlink(template) == -1)
		werror("Failed to remove temporary file: %s", strerror(errno));

	free(command);
	free(template);

	return dbuffer;
}

