/*
 * Author: Samy Al Bahra  <samy@kerneled.org>
 * Date  : Mon Dec 30 02:23:40 2002
 * Copyright(C) 2002-2003 Samy Al Bahra
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: wnews.c,v 1.6 2004/01/05 11:11:35 samy Exp $
 */

#include <errno.h>
#include <curses.h>
#include <signal.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <headline.h>
#include <config.h>
#include <rss.h>
#include <socket.h>
#include <interface.h>
#include <wstring.h>

#define VERSION "0.7"

void
usage(FILE * stdwtf)
{
	fprintf(stdwtf, " Usage: wnews [options]\n");
	fprintf(stdwtf, " Options:\n");
	fprintf(stdwtf, "    -a             Display window borders using ASCII characters\n");
	fprintf(stdwtf, "    -b             Do not display bars in text output, must used with -t\n");
	fprintf(stdwtf, "    -c <number>    Number of columns for output text to be formatted to,\n");
	fprintf(stdwtf, "                   must be used with -t\n");
	fprintf(stdwtf, "    -f             Do not do ANY formatting or parsing to recieved data.\n");
	fprintf(stdwtf, "                   Must be used -t, formatting tags ignored.\n");
	fprintf(stdwtf, "    -h             Display this message\n");
	fprintf(stdwtf, "    -i             Use interactive mode (may not be used with -t)\n");
	fprintf(stdwtf, "    -l             Do not parse HTML characters\n");
	fprintf(stdwtf, "    -n <number>    Number of the headline you would like to display, used with -t\n");
	fprintf(stdwtf, "    -p <proxy>     Use the specified proxy instead of the one found\n");
	fprintf(stdwtf, "                   in the configuration file\n");
	fprintf(stdwtf, "    -r             List all available news resources\n");
	fprintf(stdwtf, "    -t <title>     Display the latest news from the resource with\n");
	fprintf(stdwtf, "                   the specified title\n");
	fprintf(stdwtf, "    -v             Display the current wnews version\n");
	fprintf(stdwtf, "    -q             Do not display introductory wnews banner\n");
}


int
main(int argc, char *argv[])
{
	struct content *gclist, **gcarray = NULL;
	struct resource *grlist, **grarray;
	char options, *sres = NULL, *snews = NULL, *junk;
	int bflag, cflag, tflag, pflag, iflag, lflag, dflag, nflag,
	    qflag, fflag, x, soff;

	if (config() == -1) {
		fprintf(stderr, "Error in configuration file creation process, fatal error...\n");
		exit(EXIT_FAILURE);
	}
	if (cconfig() == -1) {
		fprintf(stderr, "Configuration file error, fatal error...\n");
		exit(EXIT_FAILURE);
	}
	if ((grlist = rconfig()) == NULL) {
		fprintf(stderr, "Resources configuration file error, fatal error...\n");
		exit(EXIT_FAILURE);
	}
	if (!(grarray = rhash(grlist, numr - 1))) {
		fprintf(stderr, "Could not hash resources list...\n");
		exit(EXIT_FAILURE);
	}
	bflag = cflag = nflag = tflag = pflag = dflag = qflag = iflag = lflag = fflag = 0;
	while ((options = getopt(argc, argv, "n:t:c:p:ivhdqblrfa")) != -1) {
		switch (options) {
		case 'a':
			ASCII = 1;
			break;
		case 'p':
			if (optarg && *optarg) {
				if ((proxy = strdup(optarg)) == NULL) {
					fprintf(stderr, "Could not allocate memory for proxy flag\n");
					exit(EXIT_FAILURE);
				}
			} else {
				usage(stderr);
				exit(EXIT_FAILURE);
			}
			pflag = 1;
			break;
		case 'd':
			printf("Debug mode enabled...\n");
			dflag = 1;
			break;
		case 't':
			if (optarg && *optarg) {
				if ((sres = strdup(optarg)) == NULL) {
					fprintf(stderr, "Could not allocate memory for text output flag\n");
					exit(EXIT_FAILURE);
				}
			} else {
				usage(stderr);
				exit(EXIT_FAILURE);
			}
			tflag = 1;
			interactive = 0;
			break;
		case 'n':
			if (optarg && *optarg) {
				nflag = atoi(optarg);
				if (nflag < 1) {
					fprintf(stderr, "The selected story cannot be less than 1 or greater than the current stories\n");
					exit(EXIT_FAILURE);
				}
			} else {
				usage(stderr);
				exit(EXIT_FAILURE);
			}
			break;
		case 'r':
			printf(" Available resources:\n");
			for (x = 0; x < numr; x++)
				printf("     %s\n", grarray[x]->title);
			printf("\n");
			return 0;
		case 'f':
			fflag = 1;
			break;
		case 'v':
			printf("%s\n", VERSION);
			return 0;
		case 'h':
			usage(stdout);
			return 0;
		case 'l':
			lflag = 1;
			break;
		case 'i':
			interactive = iflag = 1;
			break;
		case 'q':
			qflag = 1;
			break;
		case 'b':
			bflag = 1;
			break;
		case 'c':
			if (optarg && *optarg) {
				cflag = atoi(optarg);
				if (cflag < 7) {
					fprintf(stderr, "The column width you selected is very skinny to be applicable anywhere\n");
					exit(EXIT_FAILURE);
				}
			} else {
				usage(stderr);
				exit(EXIT_FAILURE);
			}
			break;
		default:
			usage(stderr);
			exit(EXIT_FAILURE);
		}
	}

	if (iflag && tflag) {
		fprintf(stderr, "You cannot mix interactive and text-based output options\n");
		usage(stderr);
		exit(EXIT_FAILURE);
	}
	
	if ((nflag || bflag || fflag) && (!tflag || interactive)) {
		fprintf(stderr, "You cannot mix interactive and text-based output options\n");
		usage(stderr);
		exit(EXIT_FAILURE);
	}
	
	if (!qflag) {
		printf("wnews (%s) - the web driven news client\n", VERSION);
		printf("Released under the terms of the GNU general public license\n");
		printf("Report bugs to samy@kerneled.org\n");
		printf(" Authors: Samy Al Bahra  <samy@kerneled.org>\n");
		printf("          Mohammad Susli <susli@kerneled.com>\n\n");
	}
	if (tflag || !interactive) {
		for (x = 0; x < numr; x++) {
			if (strcasecmp(grarray[x]->title, sres) == 0) {
				tflag = 0;
				soff = x;
			}
		}
		if (tflag) {
			fprintf(stderr, "The resource you requested with -t does not exist.\n");
			fprintf(stderr, " Available resources:\n");
			for (x = 0; x < numr; x++)
				fprintf(stderr, "     %s\n", grarray[x]->title);
			fprintf(stderr, "\n");
			exit(EXIT_FAILURE);
		} else {
			if (dflag)
				fprintf(stderr, "Retrieving data from HTTP server...");
			if ((snews = getpages(grarray[soff]->address, proxy)) == NULL) {
				fprintf(stderr, "Could not retrieve data from HTTP server, fatal error...\n");
				exit(EXIT_FAILURE);
			}
			if (dflag)
				fprintf(stderr, "done\n");
			if (fflag) {
				printf("%s\n", snews);
				return 0;
			}
			if (dflag)
				fprintf(stderr, "Parsing HTML data into linked list...");
			gclist = getrss(snews);
			if (dflag)
				fprintf(stderr, "done\n");

			if (!numc) {
				fprintf(stderr, "The news resource you requested has no content in it\n");
				exit(EXIT_FAILURE);
			}
			if (dflag)
				fprintf(stderr, "Hashing content list...");
			gcarray = chash(gclist, numc - 1);
			if (dflag)
				fprintf(stderr, "done\n");

			if (!lflag) {
				if (dflag)
					fprintf(stderr, "Parsing HTML characters...");
				for (x = 0; x < numc; x++) {
					gcarray[x]->desc = strhtml(gcarray[x]->desc);
				}
				if (dflag)
					fprintf(stderr, "done\n");
			}
			if (nflag) {
				if (nflag > numc) {
					fprintf(stderr, "The story you selected is out of range\n");
					exit(EXIT_FAILURE);
				}
				if (!bflag)
					printf("---------------------[URL]----------------------\n");
				printf("%s\n", cflag ? strcol(gcarray[nflag - 1]->link, cflag) : gcarray[nflag - 1]->link);
				if (!bflag)
					printf("------------------[Headline]--------------------\n");
				printf("%s\n", cflag ? strcol(gcarray[nflag - 1]->headline, cflag) : gcarray[nflag - 1]->headline);
				if (!bflag)
					printf("-----------------[Description]------------------\n");
				printf("%s\n", cflag ? strcol(gcarray[nflag - 1]->desc, cflag) : gcarray[nflag - 1]->desc);
			} else {
				for (x = 0; x < numc; x++) {
					if (!bflag)
						printf("---------------------[URL]----------------------\n");
					printf("%s\n", cflag ? strcol(gcarray[x]->link, cflag) : gcarray[x]->link);
					if (!bflag)
						printf("------------------[Headline]--------------------\n");
					printf("%s\n", cflag ? strcol(gcarray[x]->headline, cflag) : gcarray[x]->headline);
					if (!bflag)
						printf("-----------------[Description]------------------\n");
					printf("%s\n\n", cflag ? strcol(gcarray[x]->desc, cflag) : gcarray[x]->desc);
				}
			}
		}
	} else {
		if (!initi())
			return -1;

		if (signal(SIGINT, iquit) == SIG_ERR || signal(SIGWINCH, iresize) == SIG_ERR) {
			if (dflag)
				fprintf(stderr, "Could not create signal handler (%s)...\n", strerror(errno));
			werror("Could not create SIGINT and resize signal handlers...");
		}
		for (x = 0; x < numr; x++)
			strchop(grarray[x]->title, 13);

		for (;;) {
			if ((soff = rmenu(grarray, numr - 1)) != -1) {
				if (dflag)
					fprintf(stderr, "Retrieving data from HTTP server...");
				wstat("Retrieving data from HTTP server...");
				free(snews);
				if ((snews = getpages(grarray[soff]->address, proxy)) == NULL) {
					wprompt("Could not retrieve data from HTTP server");
					continue;
				}
				if (dflag)
					fprintf(stderr, "done\n");
				if (dflag)
					fprintf(stderr, "Parsing HTML data into linked list...");

				wstat("Parsing HTML data into linked list...");
				gclist = getrss(snews);
				
				if (!gclist) {
					wprompt("Could not parse HTML data");
					continue;
				}
				
				if (dflag)
					fprintf(stderr, "done\n");

				if (!numc) {
					wprompt("The news resource you requested has no content in it");
					continue;
				}
				if (dflag)
					fprintf(stderr, "Hashing content list...");
				wstat("Hashing content list...");
				gcarray = chash(gclist, numc - 1);
				if (dflag)
					fprintf(stderr, "done\n");

				if (!lflag) {
					if (dflag)
						fprintf(stderr, "Parsing HTML characters....");
					wstat("Parsing HTML characters...");
					for (x = 0; x < numc; x++) {
						gcarray[x]->link = strhtml(gcarray[x]->link);
						gcarray[x]->headline = strhtml(gcarray[x]->headline);
						gcarray[x]->desc = strcol(strhtml(gcarray[x]->desc), c - 18);
					}
					if (dflag)
						fprintf(stderr, "done\n");
				}
				hmenu(gcarray, numc);
				contfree(gcarray, numc);
			}
		}
	}

	return 0;
}
